#ifndef TRXPARTICLE_H
#define TRXPARTICLE_H
/** \file
   Definitions for particles.
*/
/*
   
   Copyright  1992-2009 M.Kraemer, GSI Darmstadt 

   $Log: trxparticle.h,v $
   Revision 1.4  2008/06/20 14:34:24  kraemer
   tilted vol's

   Revision 1.3  2008/02/05 10:55:33  kraemer
   doxygenized

   Revision 1.2  2002/11/25 16:31:04  kraemer
   record start loc,dir; particleevt w/ header; kinematics macro

   Revision 1.1  2001/01/09 14:54:35  kraemer
   *** empty log message ***


*/

#include "m.h"
#include "cmderr.h"
#include "trxrnd.h"
#include "trxxs.h"
#include "trxvol.h"
#include "trxgeo.h"
#include "trxsrc.h"
                                      /* common masks and flags */
                                      /* masks */
#define TRXPARTICLE_MASKGLOBAL 0xFF000000  /* global (e.g. debug) */
#define TRXPARTICLE_MASKFUNC   0x00FF0000  /* function specific flags */
#define TRXPARTICLE_MASKQUAL   0x0000FFFF  /* for additional qualifiers */

                                      /* flags */
#define TRXPARTICLE_DEBUG      0x80000000  /* general check flag */
#define TRXPARTICLE_CHECK      TRXPARTICLE_DEBUG   /* general check flag */
#define TRXPARTICLE_QUIET      0x10000000  /* suppress error messages */
#define TRXPARTICLE_NOMSG      TRXPARTICLE_QUIET

                                      /** \struct STRXPARTICLE
                                          \brief Describes a particle.
                                      */ 
struct STRXPARTICLE {
   unsigned long ulFlag;              /**< primary event number */    
   unsigned long ulEvt;               /**< primary event number */    
   unsigned long ulID;                /**< unique particle ID */
   unsigned long ulIASequence;        /**< interaction sequence number */ 
   long lIA;                          /**< interaction type */
   unsigned long ulVolSrc;            /**< source volume of primary particle */
   struct STRXVOL *psVol;             /**< current volume */
   struct STRXVOL *psVolExcl;         /**< volume excluded from isect */
   unsigned long ulSrf;               /**< current surface number, 1..n */
   unsigned long ulSrfExcl;           /**< surface excluded from isect */
   double dSigTot;                    /**< total x-sect for next interaction */
   double dMFP;                       /**< mean free path */
   double dDist;                      /**< distance to intersection */
   double dvx[4], dvx0[4];            /**< x,t actual, original */
   double dvd[4], dvd0[4];            /**< direction vector actual, original */                     
   double dE, dE0;                    /**< kinetic energy actual, original
                                           [keV] (x,e-,e+)
                                           [MeV/u] (ions)       
                                      */
   double dbeta2, dbeta, dgamma, dT;  /**< beta**2,..., kinetic energy [keV] */
   double ddE;                        /**< energy loss for current interaction [keV] */
   struct STRXPROJ sP;                /**< projectile */
   struct SCMDERR sCE;                /**< error handling */
};

                                      /* common macro to derive kinematics from kinetic energy */
#define TRXPARTICLEKINEMATIC(p)\
   switch((p).sP.ulType ) {\
   case TRXPROJ_ION:\
      (p).dgamma = (p).dT / ( TRX_AMU * 1.0E3 * (p).sP.sZA.dA ) + 1.0E0;\
      (p).dbeta2 = 1.0E0 - ( 1.0E0 / (p).dgamma ) * ( 1.0E0 / (p).dgamma );\
      (p).dbeta  = sqrt((p).dbeta2);\
      (p).dE = (p).dT * 1E-3 / (p).sP.sZA.dA;\
   break;\
   case TRXPROJ_ELECTRON:\
   case TRXPROJ_POSITRON:\
      (p).dgamma = (p).dT / TRX_M0 + 1.0E0;\
      (p).dbeta2 = 1.0E0 - ( 1.0E0 / (p).dgamma ) * ( 1.0E0 / (p).dgamma );\
      (p).dbeta  = sqrt((p).dbeta2);\
      (p).dE = (p).dT;\
   break;\
   case TRXPROJ_X:\
      (p).dgamma = DBL_MAX;\
      (p).dbeta2 = 1.0E0;\
      (p).dbeta  = sqrt((p).dbeta2);\
      (p).dE = (p).dT;\
   break;\
   default:\
      sCE.iRC = ERRTRXPARTICLE_BADPROJ; goto gError;\
   break;\
   }
/*

   Particle handling functions
   ===========================

   General
   -------

   TRXParticleChangeDir  ( partto, partfr, costh, phi, flag )
                         change particle direction vector
                       6 x to test
                         o calls:

   TRXParticleFree       ( part, flag ) 
                         free resources
                       3 x to test
                         o calls:
			                      o TRXProjFree()
			                      o TRXParticleInit()  
                         
   TRXParticleInit       ( part, flag ) 
                         initialize
                       2 x to test
                         o calls:
			                        o TRXProjInit()

   TRXParticleInspect    ( part, flag )
                         inspect
                       1 x to test
                         o calls:
			                        o TRXProjInspect()

   TRXParticleScore      ( particle, maskPT, maskIA, flag )
                         score interaction & eloss
                       4 x to test
                         o calls:  
   
   TRXParticle3Product   ( part1, part2, part3, flag )
                         spat-product to test co-planarity
                       5 x to test
                         o calls  
  

   TRXParticleError  : error handling

*/

int TRXParticleChangeDir( struct STRXPARTICLE *, struct STRXPARTICLE *, 
                          double, double, unsigned long );
int TRXParticleFree     ( struct STRXPARTICLE *, unsigned long );
int TRXParticleInit     ( struct STRXPARTICLE *, unsigned long );
int TRXParticleInspect  ( struct STRXPARTICLE *, unsigned long );
int TRXParticleScore    ( struct STRXPARTICLE *, unsigned long, unsigned long, unsigned long );
double TRXParticle3Product ( struct STRXPARTICLE *, struct STRXPARTICLE *, struct STRXPARTICLE *, unsigned long );

struct STRXPARTICLE *_TRXParticleTest( unsigned long ); 

                                      /** \struct STRXPARTICLEVTOLD
                                          \brief particle event output, old format.
                                      */ 
struct STRXPARTICLEEVTOLD {
   unsigned long ulEvt;               /**< primary event number */
   unsigned long ulID;                /**< particle ID */
   unsigned long ulIASequence;        /**< interaction sequence number */ 
   unsigned long ulPT;                /**< particle type */
            long  lIA;                /**< interaction */
   unsigned long ulVol;               /**< Vol ID */
   double dvx0[4];                    /**< 0..2: location, 3: time */
   float rvd [3];                     /**< direction */
   float rE, rdE;                     /**< energy, energy loss */
};


#define TRXPARTICLEEVT_FILEVERSION 20020830

                                      /** \struct STRXPARTICLEVT
                                          \brief particle event output.
                                      */ 
struct STRXPARTICLEEVT {
   unsigned long ulVersion, ulLength; /**< version, length of this structure */
   unsigned short uiByteSex;          /**< =1 for native endian, else byte swap */
   unsigned short uiMode;             /**< always=0 for particle evt */
   unsigned long ulEvt;               /**< primary event number */
   unsigned long ulID;                /**< particle ID */
   unsigned long ulIASequence;        /**< interaction sequence number */ 
   unsigned long ulPT;                /**< particle type */
            long  lIA;                /**< interaction */
   unsigned long ulVol, ulVolSrc;     /**< Vol ID actual, original */
   double dvx[4], dvx0[4];            /**< 0..2: location, 3: time; actual, original */
   float  rvd[3], rvd0[3];            /**< direction actual, original */
   float rE, rE0;                     /**< energy actual, original */
   float rdE;                         /**< energy loss */
};

/*
   Particle events for output
   --------------------------
    
   TRXParticleEvt         ( partevt, part, flag )
                          x to test
		                          o calls

   TRXParticleEvtInspect  ( partevt, flag )
                          x to test
                          o calls:


*/
                                      /* function prototypes */
int TRXParticleEvtInspect( struct STRXPARTICLEEVT *, unsigned long );
int TRXParticleEvt       ( struct STRXPARTICLEEVT *, struct STRXPARTICLE *, unsigned long );

struct STRXPARTICLEEVT *_TRXParticleEvtTest( unsigned long ); 

/*

   Functions for particles & volumes
   -------------------------------
   
   TRXParticleVolISect      ( particle, vol, flag )
                              Query particle intersection with a volume
                          4 . to test
                            o returns:
                              o  < 0: error
                              o  > 0: intersection
                              o == 0: no intersection
                            . tilted volumes
                            o calls
                              o TRXParticleVolISectSrf()
   
   TRXParticleVolISectSrf   ( particle, vol, srf, flag )
                              Query particle intersection with a surface of a volume
                          3 . to test
                              o to complete
                                o torus
                                o angular limits
                                . tilted volumes
                            o returns:
                              o  < 0: error
                              o  > 0: intersection
                              o == 0: no intersection
                            o calls
   
   TRXParticleVolLocate     ( particle, vol, flag )
                              Locate a particle within a volume
                          2 . to test
                              o to complete
                                o torus
                                o angular limits
                                . tilted volumes
                            o returns:
                              o  < 0: error
                              o  > 0: inside
                              o == 0: outside
                            o calls
                              o TRXParticleVolLocateSrf()

   TRXParticleVolLocateSrf  ( particle, vol, srf, flag )
                              Locate particle on a volume's surface
                          1 . to test 
                              o to complete 
                                o torus
                                o angular limits
                                . tilted volumes
                            o returns:
                              o  < 0: error
                              o  > 0: on surface
                              o == 0: not on surface
                            o calls
                              o TRXParticleVolLocateSrf() 

*/

                                      /* function action flags */
                                      /* don't isect recursively */ 
#define TRXPARTICLEVOL_NORECURSIVEISECT 0x00010000
                                      /* skip transformation */ 
#define TRXPARTICLEVOL_SKIPTRAFO        0x00020000

int TRXParticleVolLocateSrf( struct STRXPARTICLE *, struct STRXVOL *, unsigned long, unsigned long );
int TRXParticleVolLocate   ( struct STRXPARTICLE *, struct STRXVOL *, unsigned long );
int TRXParticleVolISectSrf ( struct STRXPARTICLE *, struct STRXVOL *, unsigned long, unsigned long );
int TRXParticleVolISect    ( struct STRXPARTICLE *, struct STRXVOL *, unsigned long );

/*

   Functions involving the geometry base
   -------------------------------------
   
   TRXParticleGeoISect   ( particle, geo, flag ); 
                         x to test
                         o calls 
                           o TRXParticleVolISect()

   TRXParticleGeoLocate  ( particle, geo, flag ); 
                         x to complete/test
                         o locate particle in volume
                         o calls 
                           o TRXParticleVolLocate()
                                   
*/

int TRXParticleGeoLocate   ( struct STRXPARTICLE *, struct STRXGEO *, unsigned long );
int TRXParticleGeoISect    ( struct STRXPARTICLE *, struct STRXGEO *, unsigned long );

/*
			 			  
   Functions for sampling of start values
   --------------------------------------

   TRXParticleSampleSrc    ( particle, src, rnd, flag )
                           x to test
                           o to complete
                             o tilted source location
                           o calls 

*/

int TRXParticleSampleSrc( struct STRXPARTICLE *, struct STRXSRC *, struct STRXRND *, unsigned long );

/*

   Functions for sampling of cross sections
   ----------------------------------------

   General: common code for locating energy / interpolation


   TRXParticleSample       ( particle, xs, ulFlag )
			                        calculate total cross section
                         1 x to test
			                        o to complete (E & sigma scaling)
                             o x
			                          o e+
			                          o ions
                           ? macro for binary search   
			                        o calls

   TRXParticleSampleA      ( particlein, particleout[2], xs, rnd, flag )
                           sample annihilation
                        10 o todo
                           o calls:                          
                             o TRXRNDVRNSD
                             o evtl: 1.5-dim inverse interpolation
                             o TRXParticleChangeDir() to change particle direction

   TRXParticleSampleB      ( particlein, particleout[2], xs, rnd, flag )
                           sample bremsstrahlung 
                         8 o todo
                           o calls:                          
                             o TRXRNDVRNSD
                             o evtl: 1.5-dim inverse interpolation
                             o TRXParticleChangeDir() to change particle direction

   TRXParticleSampleC      ( particlein, particleout[2], xs, rnd, flag )
                             sample compton effect
                         6 o todo
                           ! check coplanarity 
                   		  	   ! re-enable iRC=1 for e- created
                           o calls:                          
                             o TRXRNDVRNSD
                             o evtl: 1.5-dim inverse interpolation
                             o TRXParticleChangeDir() to change particle direction

   TRXParticleSampleF      ( particlein, particleout[2], xs, rnd, flag )
                             sample photo effect
                         7 o todo
                           ! needs info on K-Shell energy, problem for ionic materials
                           o calls:                          
                             o TRXRNDVRNSD
                             o evtl: 1.5-dim inverse interpolation
                             o TRXParticleChangeDir() to change particle direction


   TRXParticleSampleI      ( particlein, particleout[2], xs, rnd, flag )
                             sample ionization 
                         5 x to test
                     			   o to complete
			                        o proper energy/angular distributions
                           o calls:                          
                             o TRXRNDVRNSD
                             o evtl: 1.5-dim inverse interpolation
                             o TRXParticleChangeDir() to change particle direction
                          
   TRXParticleSampleL      ( particlein, particleout[2], xs, rnd, flag )
                             sample elastic scattering
                         3 x to test
			                        o to complete
                             o photons
			                          o ions
                           o calls:                          
                             o TRXRNDVRNSD
                             o evtl: 1.5-dim inverse interpolation
                             o TRXParticleChangeDir()

   TRXParticleSampleP      ( particlein, particleout[2], xs, rnd, flag )
                           sample pair production
                         9 o todo
                           o calls:                          
                             o TRXRNDVRNSD
                             o evtl: 1.5-dim inverse interpolation
                             o TRXParticleChangeDir() to change particle direction

   TRXParticleSampleX      ( particlein, particleout[2], xs, rnd, flag )
                           sample excitation 
                         4 x to test
                           o calls:                          

   TRXParticleSampleTotal  ( particle, xsb, rnd, flag );
                             for given particle type/material
			                          determine total cross section, mean free path,
			                          type of next interaction
                         2 x to test
                           o calls:
                             o TRXXSBaseTargLocate
                             o TRXParticleSample
                             o TRXRNDVRNSD  

   TRXParticleSampleIAct   ( particlein, sPOut[2], xsb, rnd, lFlagXS )
                             sample given interaction/generate secondary particles
                        11 o todo
                           o returns # of outgoing particles
                           ! others than Compton 
                           o calls:
                             o TRXXSBaseTargLocate
                             o TRXParticleSampleI()
                             o TRXParticleSampleX()
                             o TRXParticleSampleL()
                             o TRXParticleSampleB()
                             o TRXParticleSampleA()
                             o TRXParticleSampleF()
                             o TRXParticleSampleC()
                             o TRXParticleSampleP()

*/

int TRXParticleSample     ( struct STRXPARTICLE *, struct STRXXS *, unsigned long ); 
int TRXParticleSampleA    ( struct STRXPARTICLE *, struct STRXPARTICLE [2], struct STRXXS *, struct STRXRND *, unsigned long ); 
int TRXParticleSampleB    ( struct STRXPARTICLE *, struct STRXPARTICLE [2], struct STRXXS *, struct STRXRND *, unsigned long ); 
int TRXParticleSampleC    ( struct STRXPARTICLE *, struct STRXPARTICLE [2], struct STRXXS *, struct STRXRND *, unsigned long ); 
int TRXParticleSampleF    ( struct STRXPARTICLE *, struct STRXPARTICLE [2], struct STRXXS *, struct STRXRND *, unsigned long ); 
int TRXParticleSampleI    ( struct STRXPARTICLE *, struct STRXPARTICLE [2], struct STRXXS *, struct STRXRND *, unsigned long ); 
int TRXParticleSampleL    ( struct STRXPARTICLE *, struct STRXPARTICLE [2], struct STRXXS *, struct STRXRND *, unsigned long ); 
int TRXParticleSampleP    ( struct STRXPARTICLE *, struct STRXPARTICLE [2], struct STRXXS *, struct STRXRND *, unsigned long ); 
int TRXParticleSampleX    ( struct STRXPARTICLE *, struct STRXPARTICLE [2], struct STRXXS *, struct STRXRND *, unsigned long ); 
int TRXParticleSampleTotal( struct STRXPARTICLE *, struct STRXXSBASE *, struct STRXRND *, unsigned long ); 
int TRXParticleSampleIAct ( struct STRXPARTICLE *, struct STRXPARTICLE [2], struct STRXXSBASE *, struct STRXRND *, unsigned long );


                                      /* error handling */
int TRXParticleError( struct SCMDERR *, unsigned long );

enum { ERRTRXPARTICLE_SYS=1,
       ERRTRXPARTICLE_BADADDR,
       ERRTRXPARTICLE_BADCOS,
       ERRTRXPARTICLE_BADDIR,
       ERRTRXPARTICLE_NOSUCHSURFACE,
       ERRTRXPARTICLE_NOSUCHVOL,
       ERRTRXPARTICLE_ISECTFAILED,
       ERRTRXPARTICLE_LOCATEFAILED,
       ERRTRXPARTICLE_EOUTOFRANGELO,
       ERRTRXPARTICLE_EOUTOFRANGEHI,
       ERRTRXPARTICLE_BADPROJ,
       ERRTRXPARTICLE_NOMATERIAL,
       ERRTRXPARTICLE_NOSUCHIA,
       ERRTRXPARTICLE_NOTANNIH,
       ERRTRXPARTICLE_NOTBREMS,
       ERRTRXPARTICLE_NOTCOMPTON,
       ERRTRXPARTICLE_NOTPHOTO,
       ERRTRXPARTICLE_NOTIONIZATION,
       ERRTRXPARTICLE_NOTELASTIC,
       ERRTRXPARTICLE_NOTPAIR,
       ERRTRXPARTICLE_NOTEXCITATION,
};

#endif
